@@ -188,9 +188,7 @@ class QueueConsumer(DependencyProvider, ProviderCollector, ConsumerMixin):
             # so we send the same exceptions
             self._consumers_ready.send_exception(stop_exc)
 
-        _log.debug('waiting for providers to unregister %s', self)
-        self._last_provider_unregistered.wait()
-        _log.debug('all providers unregistered %s', self)
+        self.wait_for_providers()
 
         try:
             _log.debug('waiting for consumer death %s', self)
@@ -229,7 +227,7 @@ class QueueConsumer(DependencyProvider, ProviderCollector, ConsumerMixin):
             # want to remove a consumer at the same time
             # TODO: With the upcomming error handling mechanism, this needs
             # TODO: to be thought through again.
-            self.last_provider_unregistered()
+            self._last_provider_unregistered.send()
             return
 
         removed_event = Event()
